@@ -57,7 +57,7 @@ static void     gail_set_focus_widget (GtkWidget *focus_widget,
 static void     gail_set_focus_object (AtkObject *focus_obj,
                                        AtkObject *obj);
 
-static GtkWidget* focus_widget = NULL;
+GtkWidget* focus_widget = NULL;
 static GtkWidget* next_focus_widget = NULL;
 static gboolean was_deselect = FALSE;
 static GtkWidget* subsequent_focus_widget = NULL;
@@ -239,26 +239,38 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
                     }
                   widget = window->focus_widget;
                 }
-              else if (window->type == GTK_WINDOW_POPUP)
+              else if (window->type == GTK_WINDOW_POPUP) 
                 {
-                  GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
+	          if (GTK_IS_BIN (widget))
+		    {
+		      GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
 
-                  if (GTK_IS_WIDGET (child) && GTK_WIDGET_HAS_GRAB (child))
-                    {
-                      if (GTK_IS_MENU_SHELL (child))
-                        {
-                          if (GTK_MENU_SHELL (child)->active_menu_item)
-                            {
-                              /*
-                               * We have a menu which has a menu item selected
-                               * so we do not report focus on the menu.
-                               */ 
-                              return TRUE; 
-                            }
-                        }
-                      widget = child;
-                    } 
+		      if (GTK_IS_WIDGET (child) && GTK_WIDGET_HAS_GRAB (child))
+			{
+			  if (GTK_IS_MENU_SHELL (child))
+			    {
+			      if (GTK_MENU_SHELL (child)->active_menu_item)
+				{
+				  /*
+				   * We have a menu which has a menu item selected
+				   * so we do not report focus on the menu.
+				   */ 
+				  return TRUE; 
+				}
+			    }
+			  widget = child;
+			} 
+		    }
+		  else /* popup window has no children; this edge case occurs in some custom code (OOo for instance) */
+		    {
+		      return TRUE;
+		    }
                 }
+	      else /* Widget is a non-popup toplevel with no focus children; 
+		      don't emit for this case either, as it's useless */
+		{
+		  return TRUE;
+		}
             }
         }
       else
@@ -493,6 +505,8 @@ gail_switch_page_watcher (GSignalInvocationHint *ihint,
 static gint
 gail_focus_idle_handler (gpointer data)
 {
+  GDK_THREADS_ENTER();
+
   focus_notify_handler = 0;
   /*
    * The widget which was to receive focus may have been removed
@@ -500,7 +514,10 @@ gail_focus_idle_handler (gpointer data)
   if (!next_focus_widget)
     {
       if (next_focus_widget != data)
-        return FALSE;
+	{
+	  GDK_THREADS_LEAVE ();
+	  return FALSE;
+	}
     }
   else
     {
@@ -511,6 +528,7 @@ gail_focus_idle_handler (gpointer data)
     
   gail_focus_notify (data);
 
+  GDK_THREADS_LEAVE ();
   return FALSE; 
 }
 
