@@ -49,7 +49,7 @@ struct _GailBoxClass
   GailContainerClass parent_class;
 };
 
-GtkAccessible *gail_box_new(GtkWidget *widget);
+AtkObject* gail_box_new (GtkWidget *widget);
 
 #ifdef __cplusplus
 }
