@@ -1072,10 +1072,16 @@ gail_entry_paste_received (GtkClipboard *clipboard,
 static gboolean
 idle_notify_insert (gpointer data)
 {
-  GailEntry *entry = GAIL_ENTRY (data);
+  GailEntry *entry;
 
+  GDK_THREADS_ENTER ();
+
+  entry = GAIL_ENTRY (data);
   entry->insert_idle_handler = 0;
   gail_entry_notify_insert (entry);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -1156,10 +1162,16 @@ gail_entry_get_character_at_offset (AtkText *text,
 static gboolean
 idle_notify_delete (gpointer data)
 {
-  GailEntry *entry = GAIL_ENTRY (data);
+  GailEntry *entry;
 
+  GDK_THREADS_ENTER ();
+
+  entry = GAIL_ENTRY (data);
   entry->delete_idle_handler = 0;
   gail_entry_notify_delete (entry);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -1305,19 +1317,22 @@ idle_do_action (gpointer data)
   GailEntry *entry;
   GtkWidget *widget;
 
+  GDK_THREADS_ENTER ();
+
   entry = GAIL_ENTRY (data);
   entry->action_idle_handler = 0;
   widget = GTK_ACCESSIBLE (entry)->widget;
-  if (widget == NULL)
-    /*
-     * State is defunct
-     */
-    return FALSE;
-
-  if (!GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
-    return FALSE;
+  if (widget == NULL /* State is defunct */ ||
+      !GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    {
+      GDK_THREADS_LEAVE ();
+      return FALSE;
+    }
 
   gtk_widget_activate (widget);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
