@@ -56,6 +56,9 @@ static void                  menu_item_selection           (GtkItem        *item
 static gboolean              find_accel                    (GtkAccelKey    *key,
                                                             GClosure       *closure,
                                                             gpointer       data);
+static gboolean              find_accel_new                (GtkAccelKey    *key,
+                                                            GClosure       *closure,
+                                                            gpointer       data);
 
 static gpointer parent_class = NULL;
 
@@ -314,30 +317,63 @@ gail_menu_item_do_action (AtkAction *action,
     return FALSE;
 }
 
+static void
+ensure_menus_unposted (GailMenuItem *menu_item)
+{
+  AtkObject *parent;
+  GtkWidget *widget;
+
+  parent = atk_object_get_parent (ATK_OBJECT (menu_item));
+  while (parent)
+    {
+      if (GTK_IS_ACCESSIBLE (parent))
+        {
+          widget = GTK_ACCESSIBLE (parent)->widget;
+          if (GTK_IS_MENU (widget))
+            {
+              if (GTK_WIDGET_MAPPED (widget))
+                gtk_menu_shell_cancel (GTK_MENU_SHELL (widget));
+
+              return;
+            }
+        }
+      parent = atk_object_get_parent (parent);
+    }
+}
+
 static gboolean
 idle_do_action (gpointer data)
 {
   GtkWidget *item;
   GtkWidget *item_parent;
   GailMenuItem *menu_item;
+  gboolean item_mapped;
+
+  GDK_THREADS_ENTER ();
 
   menu_item = GAIL_MENU_ITEM (data);
   menu_item->action_idle_handler = 0;
   item = GTK_ACCESSIBLE (menu_item)->widget;
-  if (item == NULL)
-    /* State is defunct */
-    return FALSE;
-
-  if (!GTK_WIDGET_SENSITIVE (item) || !GTK_WIDGET_VISIBLE (item))
+  if (item == NULL /* State is defunct */ ||
+      !GTK_WIDGET_SENSITIVE (item) || !GTK_WIDGET_VISIBLE (item))
+  {
+    GDK_THREADS_LEAVE ();
     return FALSE;
+  }
 
   item_parent = gtk_widget_get_parent (item);
   gtk_menu_shell_select_item (GTK_MENU_SHELL (item_parent), item);
+  item_mapped = GTK_WIDGET_MAPPED (item);
   /*
    * This is what is called when <Return> is pressed for a menu item
    */
   g_signal_emit_by_name (item_parent, "activate_current",  
                          /*force_hide*/ 1); 
+  if (!item_mapped)
+    ensure_menus_unposted (menu_item);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -382,7 +418,9 @@ gail_menu_item_get_keybinding (AtkAction *action,
   /*
    * This function returns a string of the form A;B;C where
    * A is the keybinding for the widget; B is the keybinding to traverse
-   * from the menubar and C is the accelerator
+   * from the menubar and C is the accelerator.
+   * The items in the keybinding to traverse from the menubar are separated
+   * by ":".
    */
   GailMenuItem  *gail_menu_item;
   gchar *keybinding = NULL;
@@ -442,7 +480,10 @@ gail_menu_item_get_keybinding (AtkAction *action,
               if (key_val != GDK_VoidSymbol)
                 {
                   key = gtk_accelerator_name (key_val, mnemonic_modifier);
-                  temp_keybinding = g_strconcat (key, full_keybinding, NULL);
+                  if (full_keybinding)
+                    temp_keybinding = g_strconcat (key, ":", full_keybinding, NULL);
+                  else 
+                    temp_keybinding = g_strconcat (key, NULL);
                   if (temp_item == item)
                     {
                       item_keybinding = g_strdup (key); 
@@ -483,16 +524,39 @@ gail_menu_item_get_keybinding (AtkAction *action,
           GtkAccelKey *key;
 
           group = gtk_menu_get_accel_group (GTK_MENU (parent));
+
           if (group)
             {
               key = gtk_accel_group_find (group, find_accel, item);
+            }
+          else
+            {
+              /*
+               * If the menu item is created using GtkAction and GtkUIManager
+               * we get here.
+               */
+              key = NULL;
+              child = GTK_BIN (item)->child;
+              if (GTK_IS_ACCEL_LABEL (child))
+                {
+                  GtkAccelLabel *accel_label;
 
-              if (key)
-                {           
-                  accelerator = gtk_accelerator_name (key->accel_key,
-                                                      key->accel_mods);
+                  accel_label = GTK_ACCEL_LABEL (child);
+                  if (accel_label->accel_closure)
+                    {
+                      key = gtk_accel_group_find (accel_label->accel_group,
+                                                  find_accel_new,
+                                                  accel_label->accel_closure);
+                    }
+               
                 }
             }
+
+          if (key)
+            {           
+              accelerator = gtk_accelerator_name (key->accel_key,
+                                                  key->accel_mods);
+            }
         }
     }
   /*
@@ -603,3 +667,11 @@ find_accel (GtkAccelKey *key,
    */
   return data == (gpointer) closure->data;
 }
+
+static gboolean
+find_accel_new (GtkAccelKey *key,
+                GClosure    *closure,
+                gpointer     data)
+{
+  return data == (gpointer) closure;
+}
