@@ -1657,6 +1657,8 @@ insert_idle_handler (gpointer data)
   GailTextView *gail_text_view;
   GtkTextBuffer *buffer;
 
+  GDK_THREADS_ENTER ();
+
   gail_text_view = GAIL_TEXT_VIEW (data);
 
   g_signal_emit_by_name (data,
@@ -1677,5 +1679,8 @@ insert_idle_handler (gpointer data)
       emit_text_caret_moved (gail_text_view, get_insert_offset (buffer));
       gail_text_view->previous_selection_bound = get_selection_bound (buffer);
     }
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
